#include <SPI.h>
#include <DMD.h>
#include <TimerOne.h>
#include <Wire.h>
#include "nunchuck_funcs.h"
#include <avr/pgmspace.h>
#include "SystemFont5x7.h"

DMD dmd(1,2,1);

// Define game area
#define gameMinX 0
#define gameMaxX 31
#define gameMinY 0
#define gameMaxY 30
#define gameCentreX 16
#define gameCentreY 16

// Define score area
#define scoreMinX 0
#define scoreMaxX 31
#define scoreY 31

#define SNAKE 0
#define PONG 1
#define BREAKOUT 2
#define RACE 3
#define DEMO 4
#define GAMES 5

#define SCORE 5
#define LEVEL 6
#define GAME 7
#define OVER 8

#define BLACK 0
#define WHITE 0xFF

#define NONE 0xFF
byte gameType = NONE;
prog_char string_0[] PROGMEM = "SNAKE ";
prog_char string_1[] PROGMEM = "PONG  ";
prog_char string_2[] PROGMEM = "BREAK ";
prog_char string_3[] PROGMEM = "RACE  ";
prog_char string_4[] PROGMEM = "DEMO  ";
prog_char string_5[] PROGMEM = "SCORE ";
prog_char string_6[] PROGMEM = "LEVEL ";
prog_char string_7[] PROGMEM = "GAME  ";
prog_char string_8[] PROGMEM = "OVER  ";

PROGMEM const char *stringTable[] =
{   
  string_0,
  string_1,
  string_2,
  string_3,
  string_4,
  string_5,
  string_6,
  string_7,
  string_8,
};


int points=0;
byte level=1;
#define levelMax gameMaxX+1 // Score to go to next level
byte speed=30;

#define speakerPin 14
#define ENDTUNE 0
#define RICKTUNE 1
#define SMOOTHTUNE 2

/*--------------------------------------------------------------------------------------
 Interrupt handler for Timer1 (TimerOne) driven DMD refresh scanning, this gets
 called at the period set in Timer1.initialize();
 --------------------------------------------------------------------------------------*/
void ScanDMD()
{ 
  dmd.scanDisplayBySPI();
}

void setup()
{
   //initialize TimerOne's interrupt/CPU usage used to scan and refresh the display
  Timer1.initialize( 5000 );           //period in microseconds to call ScanDMD. Anything longer than 5000 (5ms) and you can see flicker.
  Timer1.attachInterrupt( ScanDMD );   //attach the Timer1 interrupt to ScanDMD which goes to dmd.scanDisplayBySPI()
  //clear/init the DMD pixels held in RAM
  dmd.clearScreen( 0 );   //true is normal (all pixels off), false is negative (all pixels on)
  dmd.selectFont(System5x7);
  Serial.begin(9600);
  nunchuck_power();
  nunchuck_init();
}

void loop()
{
  if (gameType == SNAKE) {
    snake_loop();
  } 
  else if (gameType==PONG) {
    pong_loop();
  } 
  else if (gameType==BREAKOUT) {
    breakout_loop();
  }
  else if (gameType==RACE) {
    race_loop();
  } 
  else if (gameType==DEMO) {
    demo_loop();
  } 
  else {
    display_intro();
  }
}

void setup_game()
{
  if (gameType == SNAKE) {
    snake_setup_game();
  } 
  else if (gameType == PONG) {
    pong_setup_game();
  } 
  else if (gameType == BREAKOUT) {
    breakout_setup_game();
  } 
  else if (gameType == RACE) {
    race_setup_game();
  } 
  else if (gameType == DEMO) {
    demo_setup_game();
  } 
  else {
    display_intro();
  }
}


void add_wall(byte x1, byte y1,byte x2, byte y2)
{
  dmd.drawLine(x1,y1,x2,y2,WHITE);
}

void update_scorebar(byte bY)
{
  Serial.print(points);
  byte mypoints = points % (levelMax);
  for (byte i=scoreMinX;i<=scoreMaxX;i++) {
    if ((i-scoreMinX) < mypoints) {
      Serial.print("#");
      dmd.writePixel(i,bY,WHITE);
    } else {
      Serial.print(".");
      dmd.writePixel(i,bY,0);
    }
  }  
  Serial.println();
}

void display_score()
{
  restart_melody();
  for (byte j=0;j<=24;j++) {
    playMelody(RICKTUNE);
    dmd.clearScreen(0);

    write_predef(LEVEL,1,1+j-24,WHITE);
    write_number(level,14,9+j-24,WHITE);

    write_predef(SCORE,1,16+j-24,WHITE);
    write_number(points,14,24+j-24,WHITE);
  }

  nunchuck_wait();
  int zbutton=0;
  while (!zbutton) {
    playMelody(RICKTUNE);
    delay(50);
    nunchuck_get_data();
    if (nunchuck_cbutton()) {
      gameType=NONE;
      zbutton=true;
    } else {
      zbutton=nunchuck_zbutton();
    }
  }
}

void display_gameover()
{
  restart_melody();
  for (byte j=gameMaxX;j>gameMinX;j--) {
    playMelody(ENDTUNE);
    dmd.scrollVert(1,false);
    write_predef(GAME,0,4+j,WHITE);
    write_predef(OVER,4,20+j,WHITE);
  }
  while(playMelody(ENDTUNE)) {}
}

void display_intro()
{
  Serial.println("Intro");
  restart_melody();
  dmd.clearScreen(0);
  byte selectDir=0;
  gameType=0;
  write_predef((gameType+GAMES-1) % GAMES,1,1,WHITE);
  dmd.drawChar(1,9,'>',WHITE,BLACK); write_predef(gameType,6,9,WHITE);
  write_predef((gameType+1) % GAMES,1,17,WHITE);
  write_predef((gameType+2) % GAMES,1,25,WHITE);
  
  boolean selected=false;
  nunchuck_wait();
  Serial.println("menu");
  while (!selected) {
    int ret = nunchuck_get_data();
    if (!ret) {
      delay(500);
      nunchuck_init();
    } else {
      playMelody(SMOOTHTUNE);
      if (nunchuck_zbutton()) {
        selected=true;
        continue;
      }

      int joyY = nunchuck_joyy();

      if(joyY < 64) {
        gameType=(gameType+1) % GAMES;

        for (int k=8; k>=0 ; k--) {
          int ret = nunchuck_get_data();
          write_predef((gameType+GAMES-2) % GAMES,1,-7+k,WHITE);
          dmd.drawChar(1,9,'>',WHITE,BLACK);
          write_predef((gameType+GAMES-1) % GAMES,map(k,0,8,1,6),1+k,WHITE);
          write_predef(gameType,map(k,0,8,6,1),9+k,WHITE);
          write_predef((gameType+1) % GAMES,1,17+k,WHITE);
          write_predef((gameType+2) % GAMES,1,25+k,WHITE);
        }
        nunchuck_wait();
      } else if (joyY > 172) {
        gameType=(gameType+GAMES-1) % GAMES;
        for (int k=-7; k<=0 ; k++) {
          int ret = nunchuck_get_data();
          write_predef((gameType+GAMES-2) % GAMES,1,-7+k,WHITE);
          write_predef((gameType+GAMES-1) % GAMES,1,1+k,WHITE);
          dmd.drawChar(1,9,'>',WHITE,BLACK);
          write_predef(gameType,map(k,-7,0,1,6),9+k,WHITE);
          write_predef((gameType+1) % GAMES,map(k,-7,0,6,1),17+k,WHITE);
          write_predef((gameType+2) % GAMES,1,25+k,WHITE);
        }
        nunchuck_wait();
      }
      delay(50);
    }
  }
  randomSeed(millis()); // Gives a pretty good random value
  setup_game();
}

void write_number(int num, int x, int y, byte brightness)
{
  char numChar[10];
  itoa(num, numChar, 10);
  write_string(numChar,x,y,brightness);
}

void write_string(char string[], int x, int y, byte brightness)
{
  byte i=0;
  while (string[i] != '\0') {
    i++;
  }
  dmd.drawString(x,y,string,i,brightness,BLACK);
}

void write_predef(byte stringNumber, int x, int y, byte brightness)
{
  char buffer[8];
  strcpy_P(buffer, (char*)pgm_read_word(&(stringTable[stringNumber])));
  write_string(buffer,x,y,brightness);
}
